==> org-lsst-ccs-subsystem-utility-gui-1.0.3-sources.jar: org/lsst/ccs/subsystem/utility/ui/jas/UTOptionalPage.java <==
package org.lsst.ccs.subsystem.utility.ui.jas;

import java.awt.BorderLayout;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import javax.swing.JComponent;
import javax.swing.JPanel;
import org.lsst.ccs.bus.data.AgentInfo;
import org.lsst.ccs.bus.messages.StatusSubsystemData;
import org.lsst.ccs.gconsole.base.Console;
import org.lsst.ccs.gconsole.base.OptionalPage;
import org.lsst.ccs.gconsole.plugins.monitor.SectionedTableView;
import org.lsst.ccs.gconsole.plugins.monitor.TabbedView;
import org.lsst.ccs.messaging.AgentMessagingLayer;

/**
 * @author tonyj
 */
public class UTOptionalPage implements OptionalPage {

    private final CommandSender cmdSender = new CommandSender();
    private final Map<String, Control> controls = new ConcurrentHashMap<>();
    private final String agentProperty;
    private final Class controlClass;

    public UTOptionalPage(String agentProperty, Class controlClass) {
        this.agentProperty = agentProperty;
        this.controlClass = controlClass;
    }

    @Override
    public boolean isAutoOpen() {
        return false;
    }

    @Override
    public String getPage(AgentInfo agent) {
        return agent != null && agent.hasAgentProperty(agentProperty)
                 ? getPageName(agent) : null;
    }

    @Override
    public boolean agentDisconnected(AgentInfo agent) {
        //System.out.println("agentDisconnected called");
        Control control = controls.get(getPageName(agent));
        if (control == null) return true;
        control.disableSystem();
        return false;
    }

    @Override
    public JComponent open(AgentInfo agent, JComponent existingComponent) {
        //System.out.println("open called");
        String pageName = getPageName(agent);
        String agentName = agent.getName();
        Control control = controls.get(pageName);
        if (control != null) {
            initControls(agentName, control);
            return existingComponent;
        }

        // Create the control object
        try {
            control = (Control)controlClass.newInstance();
            controls.put(pageName, control);
        }
        catch (Exception e) {
            return null;
        }

        // Create the monitoring panel
        TabbedView view = new TabbedView();
        view.setViewFactory(name -> new SectionedTableView());            
        MonitorFilter monitorFilter = new MonitorFilter(agent.getName());
        view.setFilter(monitorFilter);
        view.install();

        // Create the control panel
        JPanel cPanel = control.getControlPanel(cmdSender);
        control.setSubsystems(agent.getName());

        // Create the whole panel
        JPanel panel = new JPanel();
        panel.setLayout(new java.awt.BorderLayout());
        panel.add(view.getPanel(), BorderLayout.CENTER);
        panel.add(cPanel, BorderLayout.SOUTH);

        // Set up the bus listener
        AgentMessagingLayer aml = Console.getConsole().getMessagingAccess();
        aml.addStatusMessageListener(control.getListener(),
                                     (msg) -> msg.getOriginAgentInfo().getName().equals(agentName)
                                                && msg instanceof StatusSubsystemData);

        // Initialize the control panel
        initControls(agentName, control);
        return panel;
    }

    @Override
    public void closed(String pageName) {
        //System.out.println("closed called");
        Control control = controls.remove(pageName);
        if (control == null) {
            throw new RuntimeException("No page exists with the name " + pageName);
        }
        Console.getConsole().getMessagingAccess().removeStatusMessageListener(control.getListener());
    }

    private String getPageName(AgentInfo agent) {
        return "CCS Subsystems/" + agent.getName();
    }

    private void initControls(String agentName, Control control) {
        control.updateControlPanel(cmdSender.sendCommand(agentName, null, "getSystemState"));
    }

}

==> org-lsst-ccs-subsystem-utility-gui-1.1.2-sources.jar: org/lsst/ccs/subsystem/utility/ui/jas/UTOptionalPage.java <==
package org.lsst.ccs.subsystem.utility.ui.jas;

import java.awt.BorderLayout;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import javax.swing.JComponent;
import javax.swing.JPanel;
import org.lsst.ccs.bus.data.AgentInfo;
import org.lsst.ccs.bus.messages.StatusSubsystemData;
import org.lsst.ccs.gconsole.base.Console;
import org.lsst.ccs.gconsole.base.OptionalPage;
import org.lsst.ccs.gconsole.plugins.monitor.SectionedTableView;
import org.lsst.ccs.gconsole.plugins.monitor.TabbedView;
import org.lsst.ccs.messaging.AgentMessagingLayer;

/**
 * @author tonyj
 */
public class UTOptionalPage implements OptionalPage {

    private final Map<String, Control> controls = new ConcurrentHashMap<>();
    private final String agentProperty;
    private final Class controlClass;

    public UTOptionalPage(String agentProperty, Class controlClass) {
        this.agentProperty = agentProperty;
        this.controlClass = controlClass;
    }

    @Override
    public boolean isAutoOpen() {
        return false;
    }

    @Override
    public String getPage(AgentInfo agent) {
        return agent != null && agent.hasAgentProperty(agentProperty)
                 ? getPageName(agent) : null;
    }

    @Override
    public boolean agentDisconnected(AgentInfo agent) {
        //System.out.println("agentDisconnected called");
        Control control = controls.get(getPageName(agent));
        if (control == null) return true;
        control.disableSystem();
        return false;
    }

    @Override
    public JComponent open(AgentInfo agent, JComponent existingComponent) {
        //System.out.println("open called");
        String pageName = getPageName(agent);
        String agentName = agent.getName();
        Control control = controls.get(pageName);
        if (control != null) {
            control.initControlPanel();
            return existingComponent;
        }

        // Create the control object
        try {
            control = (Control)controlClass.newInstance();
            controls.put(pageName, control);
        }
        catch (Exception e) {
            return null;
        }

        // Create the monitoring panel
        TabbedView view = new TabbedView();
        view.setViewFactory(name -> new SectionedTableView());            
        view.setFilter(new MonitorFilter(agentName));
        view.install();

        // Create the control panel
        JPanel cPanel = control.getControlPanel(new CommandSender(agentName));

        // Create the whole panel
        JPanel panel = new JPanel();
        panel.setLayout(new java.awt.BorderLayout());
        panel.add(view.getPanel(), BorderLayout.CENTER);
        panel.add(cPanel, BorderLayout.SOUTH);

        // Set up the bus listener
        AgentMessagingLayer aml = Console.getConsole().getMessagingAccess();
        aml.addStatusMessageListener(control.getListener(),
                                     (msg) -> msg.getOriginAgentInfo().getName().equals(agentName)
                                                && msg instanceof StatusSubsystemData);

        // Initialize the control panel
        control.initControlPanel();
        return panel;
    }

    @Override
    public void closed(String pageName) {
        //System.out.println("closed called");
        Control control = controls.remove(pageName);
        if (control == null) {
            throw new RuntimeException("No page exists with the name " + pageName);
        }
        Console.getConsole().getMessagingAccess().removeStatusMessageListener(control.getListener());
    }

    private String getPageName(AgentInfo agent) {
        return "CCS Subsystems/" + agent.getName();
    }

}
